/*
 *  Licensed Materials - Property of IBM
 *  5725-G92 (C) Copyright IBM Corp. 2006, 2013. All Rights Reserved.
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.prov;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletResponse;

import com.ibm.json.java.JSONObject;
import com.worklight.core.auth.ext.DeviceAutoProvisioningAuthenticator;
import com.worklight.server.auth.api.AuthenticationResult;
import com.worklight.server.auth.api.MissingConfigurationOptionException;

public class MyProvisioningAuthenticator extends DeviceAutoProvisioningAuthenticator {
	
	private static URL MY_URL;

	@Override
	public void init(Map<String, String> option) throws MissingConfigurationOptionException {
		entityString = option.remove(PROVISIONED_ENTITY_PARAM_NAME);
		preRequiredRealms = option.remove(PRE_REQUIRED_REALMS_PARAM_NAME);
		super.init(option);
		
		try {
			MY_URL = new URL("http://localhost:8089/");
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}		
	}
	
	@Override
	protected AuthenticationResult checkChallangeResponse(Object challengeResponse, HttpServletResponse response) throws IOException {
		if(challengeResponse instanceof JSONObject) {
			JSONObject challengeJSON = (JSONObject) challengeResponse;
			
			if (challengeJSON.containsKey(CSR_PARAM_NAME)) {
				if (isProvisioningAllowed()) {
					return handleCSR((String) challengeJSON.get(CSR_PARAM_NAME), response);
				} else {
					return AuthenticationResult.createFailureResult(new JSONObject(), "Provisioning is not allowed at this time");
				}

			}
		}
		return super.checkChallangeResponse(challengeResponse, response);
	}

	private AuthenticationResult handleCSR(String csr, HttpServletResponse response) {
		AuthenticationResult result;
		try {
		    URLConnection connection = MY_URL.openConnection();
		    connection.setDoOutput(true);
		    connection.setDoInput(true);
		    connection.connect();
		    
		    OutputStreamWriter osw = new OutputStreamWriter(connection.getOutputStream());

		    osw.write(csr);
		    osw.close();

		    BufferedReader br = new BufferedReader(new InputStreamReader(connection.getInputStream()));
		    
		    StringBuffer sb = new StringBuffer();
		    String s;
		    while((s = br.readLine()) != null) {
		    	sb.append(s);
		    }

		    br.close();
		    result = createNewChallenge();
		    result.getJson().put(CERTIFICATE, sb.toString());
		}
		catch (IOException e) {   
			result = AuthenticationResult.createFailureResult(new JSONObject(), e.getMessage());
		}
		return result;
	}
	
}
